@@ -11,10 +11,62 @@
 #include "xss-common-qsort.h"
 #include "xss-network-keyvaluesort.hpp"
 
-#if defined(XSS_USE_OPENMP) && defined(_OPENMP)
-#define XSS_COMPILE_OPENMP
-#include <omp.h>
-#endif
+/*
+ * Sort all the NAN's to end of the array and return the index of the last elem
+ * in the array which is not a nan
+ */
+template <typename T1, typename T2, typename vtype>
+X86_SIMD_SORT_INLINE arrsize_t move_nans_to_end_of_array(T1 *keys,
+                                                         T2 *vals,
+                                                         arrsize_t size)
+{
+    using reg_t = typename vtype::reg_t;
+
+    arrsize_t jj = size - 1;
+    arrsize_t ii = 0;
+    arrsize_t count = 0;
+
+    while (ii + vtype::numlanes < jj) {
+        reg_t in = vtype::loadu(keys + ii);
+        auto nanmask = vtype::convert_mask_to_int(
+                vtype::template fpclass<0x01 | 0x80>(in));
+
+        // Check if there are any nans in this vector, and process them if so
+        if (nanmask != 0x00) {
+            for (size_t offset = 0; offset < vtype::numlanes; offset++) {
+                if (is_a_nan(keys[ii])) {
+                    std::swap(keys[ii], keys[jj]);
+                    std::swap(vals[ii], vals[jj]);
+                    jj -= 1;
+                    count++;
+                }
+                else {
+                    ii += 1;
+                }
+            }
+        }
+        else {
+            ii += vtype::numlanes;
+        }
+    }
+
+    // Handle the remainders once we have less than 1 vector worth
+    while (ii < jj) {
+        if (is_a_nan(keys[ii])) {
+            std::swap(keys[ii], keys[jj]);
+            std::swap(vals[ii], vals[jj]);
+            jj -= 1;
+            count++;
+        }
+        else {
+            ii += 1;
+        }
+    }
+
+    /* Haven't checked for nan when ii == jj */
+    if (is_a_nan(keys[ii])) { count++; }
+    return size - count - 1;
+}
 
 /*
  * Parition one ZMM register based on the pivot and returns the index of the
@@ -72,7 +124,7 @@ X86_SIMD_SORT_INLINE arrsize_t kvpartition(type_t1 *keys,
     for (int32_t i = (right - left) % vtype1::numlanes; i > 0; --i) {
         *smallest = std::min(*smallest, keys[left]);
         *biggest = std::max(*biggest, keys[left]);
-        if (keys[left] > pivot) {
+        if (keys[left] >= pivot) {
             right--;
             std::swap(keys[left], keys[right]);
             std::swap(indexes[left], indexes[right]);
@@ -204,12 +256,13 @@ X86_SIMD_SORT_INLINE arrsize_t kvpartition_unrolled(type_t1 *keys,
         return kvpartition<vtype1, vtype2>(
                 keys, indexes, left, right, pivot, smallest, biggest);
     }
+
     /* make array length divisible by vtype1::numlanes , shortening the array */
     for (int32_t i = ((right - left) % (num_unroll * vtype1::numlanes)); i > 0;
          --i) {
         *smallest = std::min(*smallest, keys[left]);
         *biggest = std::max(*biggest, keys[left]);
-        if (keys[left] > pivot) {
+        if (keys[left] >= pivot) {
             right--;
             std::swap(keys[left], keys[right]);
             std::swap(indexes[left], indexes[right]);
@@ -386,18 +439,27 @@ X86_SIMD_SORT_INLINE void kvsort_(type1_t *keys,
      * Base case: use bitonic networks to sort arrays <= 128
      */
     if (right + 1 - left <= 128) {
-
         kvsort_n<vtype1, vtype2, 128>(
                 keys + left, indexes + left, (int32_t)(right + 1 - left));
         return;
     }
 
-    type1_t pivot = get_pivot_blocks<vtype1>(keys, left, right);
+    // Ascending comparator for this vtype
+    using comparator = Comparator<vtype1, false>;
+    type1_t pivot;
+    auto pivot_result
+            = get_pivot_smart<vtype1, comparator, type1_t>(keys, left, right);
+    pivot = pivot_result.pivot;
+
+    if (pivot_result.result == pivot_result_t::Sorted) { return; }
+
     type1_t smallest = vtype1::type_max();
     type1_t biggest = vtype1::type_min();
     arrsize_t pivot_index = kvpartition_unrolled<vtype1, vtype2, 4>(
             keys, indexes, left, right + 1, pivot, &smallest, &biggest);
 
+    if (pivot_result.result == pivot_result_t::Only2Values) { return; }
+
 #ifdef XSS_COMPILE_OPENMP
     if (pivot != smallest) {
         bool parallel_left = (pivot_index - left) > task_threshold;
@@ -519,6 +581,9 @@ X86_SIMD_SORT_INLINE void xss_qsort_kv(
                                       half_vector<T2>,
                                       full_vector<T2>>::type;
 
+    // Exit early if no work would be done
+    if (arrsize <= 1) return;
+
 #ifdef XSS_TEST_KEYVALUE_BASE_CASE
     int maxiters = -1;
     bool minarrsize = true;
@@ -528,11 +593,12 @@ X86_SIMD_SORT_INLINE void xss_qsort_kv(
 #endif // XSS_TEST_KEYVALUE_BASE_CASE
 
     if (minarrsize) {
-        arrsize_t nan_count = 0;
+        arrsize_t index_last_elem = arrsize - 1;
         if constexpr (xss::fp::is_floating_point_v<T1>) {
             if (UNLIKELY(hasnan)) {
-                nan_count
-                        = replace_nan_with_inf<full_vector<T1>>(keys, arrsize);
+                index_last_elem
+                        = move_nans_to_end_of_array<T1, T2, full_vector<T1>>(
+                                keys, indexes, arrsize);
             }
         }
         else {
@@ -544,9 +610,7 @@ X86_SIMD_SORT_INLINE void xss_qsort_kv(
         bool use_parallel = arrsize > 10000;
 
         if (use_parallel) {
-            // This thread limit was determined experimentally; it may be better for it to be the number of physical cores on the system
-            constexpr int thread_limit = 8;
-            int thread_count = std::min(thread_limit, omp_get_max_threads());
+            int thread_count = xss_get_num_threads();
             arrsize_t task_threshold
                     = std::max((arrsize_t)10000, arrsize / 100);
 
@@ -555,24 +619,37 @@ X86_SIMD_SORT_INLINE void xss_qsort_kv(
             // Note that we do not use the if(...) clause built into OpenMP, because it causes a performance regression for small arrays
 #pragma omp parallel num_threads(thread_count)
 #pragma omp single
-            kvsort_<keytype, valtype>(
-                    keys, indexes, 0, arrsize - 1, maxiters, task_threshold);
+            kvsort_<keytype, valtype>(keys,
+                                      indexes,
+                                      0,
+                                      index_last_elem,
+                                      maxiters,
+                                      task_threshold);
+#pragma omp taskwait
         }
         else {
-            kvsort_<keytype, valtype>(
-                    keys, indexes, 0, arrsize - 1, maxiters, 0);
+            kvsort_<keytype, valtype>(keys,
+                                      indexes,
+                                      0,
+                                      index_last_elem,
+                                      maxiters,
+                                      std::numeric_limits<arrsize_t>::max());
         }
 #else
-        kvsort_<keytype, valtype>(keys, indexes, 0, arrsize - 1, maxiters, 0);
+        kvsort_<keytype, valtype>(
+                keys, indexes, 0, index_last_elem, maxiters, 0);
 #endif
 
-        replace_inf_with_nan(keys, arrsize, nan_count);
-
         if (descending) {
             std::reverse(keys, keys + arrsize);
             std::reverse(indexes, indexes + arrsize);
         }
     }
+
+#ifdef __MMX__
+    // Workaround for compiler bug generating MMX instructions without emms
+    _mm_empty();
+#endif
 }
 
 template <typename T1,
@@ -599,6 +676,9 @@ X86_SIMD_SORT_INLINE void xss_select_kv(T1 *keys,
                                       half_vector<T2>,
                                       full_vector<T2>>::type;
 
+    // Exit early if no work would be done
+    if (arrsize <= 1) return;
+
 #ifdef XSS_TEST_KEYVALUE_BASE_CASE
     int maxiters = -1;
     bool minarrsize = true;
@@ -610,20 +690,19 @@ X86_SIMD_SORT_INLINE void xss_select_kv(T1 *keys,
     if (minarrsize) {
         if (descending) { k = arrsize - 1 - k; }
 
-        if constexpr (std::is_floating_point_v<T1>) {
-            arrsize_t nan_count = 0;
+        arrsize_t index_last_elem = arrsize - 1;
+        if constexpr (xss::fp::is_floating_point_v<T1>) {
             if (UNLIKELY(hasnan)) {
-                nan_count
-                        = replace_nan_with_inf<full_vector<T1>>(keys, arrsize);
+                index_last_elem
+                        = move_nans_to_end_of_array<T1, T2, full_vector<T1>>(
+                                keys, indexes, arrsize);
             }
-            kvselect_<keytype, valtype>(
-                    keys, indexes, k, 0, arrsize - 1, maxiters);
-            replace_inf_with_nan(keys, arrsize, nan_count);
         }
-        else {
-            UNUSED(hasnan);
+
+        UNUSED(hasnan);
+        if (index_last_elem >= k) {
             kvselect_<keytype, valtype>(
-                    keys, indexes, k, 0, arrsize - 1, maxiters);
+                    keys, indexes, k, 0, index_last_elem, maxiters);
         }
 
         if (descending) {
@@ -631,6 +710,11 @@ X86_SIMD_SORT_INLINE void xss_select_kv(T1 *keys,
             std::reverse(indexes, indexes + arrsize);
         }
     }
+
+#ifdef __MMX__
+    // Workaround for compiler bug generating MMX instructions without emms
+    _mm_empty();
+#endif
 }
 
 template <typename T1,
