@@ -7,16 +7,19 @@ import numpy as np
 import numpy.polynomial.laguerre as lag
 from numpy.polynomial.polynomial import polyval
 from numpy.testing import (
-    assert_almost_equal, assert_raises, assert_equal, assert_,
-    )
+    assert_,
+    assert_almost_equal,
+    assert_equal,
+    assert_raises,
+)
 
-L0 = np.array([1])/1
-L1 = np.array([1, -1])/1
-L2 = np.array([2, -4, 1])/2
-L3 = np.array([6, -18, 9, -1])/6
-L4 = np.array([24, -96, 72, -16, 1])/24
-L5 = np.array([120, -600, 600, -200, 25, -1])/120
-L6 = np.array([720, -4320, 5400, -2400, 450, -36, 1])/720
+L0 = np.array([1]) / 1
+L1 = np.array([1, -1]) / 1
+L2 = np.array([2, -4, 1]) / 2
+L3 = np.array([6, -18, 9, -1]) / 6
+L4 = np.array([24, -96, 72, -16, 1]) / 24
+L5 = np.array([120, -600, 600, -200, 25, -1]) / 120
+L6 = np.array([720, -4320, 5400, -2400, 450, -36, 1]) / 720
 
 Llist = [L0, L1, L2, L3, L4, L5, L6]
 
@@ -50,7 +53,7 @@ class TestArithmetic:
                 tgt = np.zeros(max(i, j) + 1)
                 tgt[i] += 1
                 tgt[j] += 1
-                res = lag.lagadd([0]*i + [1], [0]*j + [1])
+                res = lag.lagadd([0] * i + [1], [0] * j + [1])
                 assert_equal(trim(res), trim(tgt), err_msg=msg)
 
     def test_lagsub(self):
@@ -60,37 +63,37 @@ class TestArithmetic:
                 tgt = np.zeros(max(i, j) + 1)
                 tgt[i] += 1
                 tgt[j] -= 1
-                res = lag.lagsub([0]*i + [1], [0]*j + [1])
+                res = lag.lagsub([0] * i + [1], [0] * j + [1])
                 assert_equal(trim(res), trim(tgt), err_msg=msg)
 
     def test_lagmulx(self):
         assert_equal(lag.lagmulx([0]), [0])
         assert_equal(lag.lagmulx([1]), [1, -1])
         for i in range(1, 5):
-            ser = [0]*i + [1]
-            tgt = [0]*(i - 1) + [-i, 2*i + 1, -(i + 1)]
+            ser = [0] * i + [1]
+            tgt = [0] * (i - 1) + [-i, 2 * i + 1, -(i + 1)]
             assert_almost_equal(lag.lagmulx(ser), tgt)
 
     def test_lagmul(self):
         # check values of result
         for i in range(5):
-            pol1 = [0]*i + [1]
+            pol1 = [0] * i + [1]
             val1 = lag.lagval(self.x, pol1)
             for j in range(5):
                 msg = f"At i={i}, j={j}"
-                pol2 = [0]*j + [1]
+                pol2 = [0] * j + [1]
                 val2 = lag.lagval(self.x, pol2)
                 pol3 = lag.lagmul(pol1, pol2)
                 val3 = lag.lagval(self.x, pol3)
                 assert_(len(pol3) == i + j + 1, msg)
-                assert_almost_equal(val3, val1*val2, err_msg=msg)
+                assert_almost_equal(val3, val1 * val2, err_msg=msg)
 
     def test_lagdiv(self):
         for i in range(5):
             for j in range(5):
                 msg = f"At i={i}, j={j}"
-                ci = [0]*i + [1]
-                cj = [0]*j + [1]
+                ci = [0] * i + [1]
+                cj = [0] * j + [1]
                 tgt = lag.lagadd(ci, cj)
                 quo, rem = lag.lagdiv(tgt, ci)
                 res = lag.lagadd(lag.lagmul(quo, ci), rem)
@@ -101,7 +104,7 @@ class TestArithmetic:
             for j in range(5):
                 msg = f"At i={i}, j={j}"
                 c = np.arange(i + 1)
-                tgt = reduce(lag.lagmul, [c]*j, np.array([1]))
+                tgt = reduce(lag.lagmul, [c] * j, np.array([1]))
                 res = lag.lagpow(c, j)
                 assert_equal(trim(res), trim(tgt), err_msg=msg)
 
@@ -113,25 +116,25 @@ class TestEvaluation:
     c3d = np.einsum('i,j,k->ijk', c1d, c1d, c1d)
 
     # some random values in [-1, 1)
-    x = np.random.random((3, 5))*2 - 1
+    x = np.random.random((3, 5)) * 2 - 1
     y = polyval(x, [1., 2., 3.])
 
     def test_lagval(self):
-        #check empty input
+        # check empty input
         assert_equal(lag.lagval([], [1]).size, 0)
 
-        #check normal input)
+        # check normal input)
         x = np.linspace(-1, 1)
         y = [polyval(x, c) for c in Llist]
         for i in range(7):
             msg = f"At i={i}"
             tgt = y[i]
-            res = lag.lagval(x, [0]*i + [1])
+            res = lag.lagval(x, [0] * i + [1])
             assert_almost_equal(res, tgt, err_msg=msg)
 
-        #check that shape is preserved
+        # check that shape is preserved
         for i in range(3):
-            dims = [2]*i
+            dims = [2] * i
             x = np.zeros(dims)
             assert_equal(lag.lagval(x, [1]).shape, dims)
             assert_equal(lag.lagval(x, [1, 0]).shape, dims)
@@ -141,15 +144,15 @@ class TestEvaluation:
         x1, x2, x3 = self.x
         y1, y2, y3 = self.y
 
-        #test exceptions
+        # test exceptions
         assert_raises(ValueError, lag.lagval2d, x1, x2[:2], self.c2d)
 
-        #test values
-        tgt = y1*y2
+        # test values
+        tgt = y1 * y2
         res = lag.lagval2d(x1, x2, self.c2d)
         assert_almost_equal(res, tgt)
 
-        #test shape
+        # test shape
         z = np.ones((2, 3))
         res = lag.lagval2d(z, z, self.c2d)
         assert_(res.shape == (2, 3))
@@ -158,15 +161,15 @@ class TestEvaluation:
         x1, x2, x3 = self.x
         y1, y2, y3 = self.y
 
-        #test exceptions
+        # test exceptions
         assert_raises(ValueError, lag.lagval3d, x1, x2, x3[:2], self.c3d)
 
-        #test values
-        tgt = y1*y2*y3
+        # test values
+        tgt = y1 * y2 * y3
         res = lag.lagval3d(x1, x2, x3, self.c3d)
         assert_almost_equal(res, tgt)
 
-        #test shape
+        # test shape
         z = np.ones((2, 3))
         res = lag.lagval3d(z, z, z, self.c3d)
         assert_(res.shape == (2, 3))
@@ -175,29 +178,29 @@ class TestEvaluation:
         x1, x2, x3 = self.x
         y1, y2, y3 = self.y
 
-        #test values
+        # test values
         tgt = np.einsum('i,j->ij', y1, y2)
         res = lag.laggrid2d(x1, x2, self.c2d)
         assert_almost_equal(res, tgt)
 
-        #test shape
+        # test shape
         z = np.ones((2, 3))
         res = lag.laggrid2d(z, z, self.c2d)
-        assert_(res.shape == (2, 3)*2)
+        assert_(res.shape == (2, 3) * 2)
 
     def test_laggrid3d(self):
         x1, x2, x3 = self.x
         y1, y2, y3 = self.y
 
-        #test values
+        # test values
         tgt = np.einsum('i,j,k->ijk', y1, y2, y3)
         res = lag.laggrid3d(x1, x2, x3, self.c3d)
         assert_almost_equal(res, tgt)
 
-        #test shape
+        # test shape
         z = np.ones((2, 3))
         res = lag.laggrid3d(z, z, z, self.c3d)
-        assert_(res.shape == (2, 3)*3)
+        assert_(res.shape == (2, 3) * 3)
 
 
 class TestIntegral:
@@ -213,15 +216,15 @@ class TestIntegral:
 
         # test integration of zero polynomial
         for i in range(2, 5):
-            k = [0]*(i - 2) + [1]
+            k = [0] * (i - 2) + [1]
             res = lag.lagint([0], m=i, k=k)
             assert_almost_equal(res, [1, -1])
 
         # check single integration with integration constant
         for i in range(5):
             scl = i + 1
-            pol = [0]*i + [1]
-            tgt = [i] + [0]*i + [1/scl]
+            pol = [0] * i + [1]
+            tgt = [i] + [0] * i + [1 / scl]
             lagpol = lag.poly2lag(pol)
             lagint = lag.lagint(lagpol, m=1, k=[i])
             res = lag.lag2poly(lagint)
@@ -230,7 +233,7 @@ class TestIntegral:
         # check single integration with integration constant and lbnd
         for i in range(5):
             scl = i + 1
-            pol = [0]*i + [1]
+            pol = [0] * i + [1]
             lagpol = lag.poly2lag(pol)
             lagint = lag.lagint(lagpol, m=1, k=[i], lbnd=-1)
             assert_almost_equal(lag.lagval(-1, lagint), i)
@@ -238,8 +241,8 @@ class TestIntegral:
         # check single integration with integration constant and scaling
         for i in range(5):
             scl = i + 1
-            pol = [0]*i + [1]
-            tgt = [i] + [0]*i + [2/scl]
+            pol = [0] * i + [1]
+            tgt = [i] + [0] * i + [2 / scl]
             lagpol = lag.poly2lag(pol)
             lagint = lag.lagint(lagpol, m=1, k=[i], scl=2)
             res = lag.lag2poly(lagint)
@@ -248,7 +251,7 @@ class TestIntegral:
         # check multiple integrations with default k
         for i in range(5):
             for j in range(2, 5):
-                pol = [0]*i + [1]
+                pol = [0] * i + [1]
                 tgt = pol[:]
                 for k in range(j):
                     tgt = lag.lagint(tgt, m=1)
@@ -258,7 +261,7 @@ class TestIntegral:
         # check multiple integrations with defined k
         for i in range(5):
             for j in range(2, 5):
-                pol = [0]*i + [1]
+                pol = [0] * i + [1]
                 tgt = pol[:]
                 for k in range(j):
                     tgt = lag.lagint(tgt, m=1, k=[k])
@@ -268,7 +271,7 @@ class TestIntegral:
         # check multiple integrations with lbnd
         for i in range(5):
             for j in range(2, 5):
-                pol = [0]*i + [1]
+                pol = [0] * i + [1]
                 tgt = pol[:]
                 for k in range(j):
                     tgt = lag.lagint(tgt, m=1, k=[k], lbnd=-1)
@@ -278,7 +281,7 @@ class TestIntegral:
         # check multiple integrations with scaling
         for i in range(5):
             for j in range(2, 5):
-                pol = [0]*i + [1]
+                pol = [0] * i + [1]
                 tgt = pol[:]
                 for k in range(j):
                     tgt = lag.lagint(tgt, m=1, k=[k], scl=2)
@@ -311,21 +314,21 @@ class TestDerivative:
 
         # check that zeroth derivative does nothing
         for i in range(5):
-            tgt = [0]*i + [1]
+            tgt = [0] * i + [1]
             res = lag.lagder(tgt, m=0)
             assert_equal(trim(res), trim(tgt))
 
         # check that derivation is the inverse of integration
         for i in range(5):
             for j in range(2, 5):
-                tgt = [0]*i + [1]
+                tgt = [0] * i + [1]
                 res = lag.lagder(lag.lagint(tgt, m=j), m=j)
                 assert_almost_equal(trim(res), trim(tgt))
 
         # check derivation with scaling
         for i in range(5):
             for j in range(2, 5):
-                tgt = [0]*i + [1]
+                tgt = [0] * i + [1]
                 res = lag.lagder(lag.lagint(tgt, m=j, scl=2), m=j, scl=.5)
                 assert_almost_equal(trim(res), trim(tgt))
 
@@ -344,7 +347,7 @@ class TestDerivative:
 
 class TestVander:
     # some random values in [-1, 1)
-    x = np.random.random((3, 5))*2 - 1
+    x = np.random.random((3, 5)) * 2 - 1
 
     def test_lagvander(self):
         # check for 1d x
@@ -352,7 +355,7 @@ class TestVander:
         v = lag.lagvander(x, 3)
         assert_(v.shape == (3, 4))
         for i in range(4):
-            coef = [0]*i + [1]
+            coef = [0] * i + [1]
             assert_almost_equal(v[..., i], lag.lagval(x, coef))
 
         # check for 2d x
@@ -360,7 +363,7 @@ class TestVander:
         v = lag.lagvander(x, 3)
         assert_(v.shape == (3, 2, 4))
         for i in range(4):
-            coef = [0]*i + [1]
+            coef = [0] * i + [1]
             assert_almost_equal(v[..., i], lag.lagval(x, coef))
 
     def test_lagvander2d(self):
@@ -394,7 +397,7 @@ class TestFitting:
 
     def test_lagfit(self):
         def f(x):
-            return x*(x - 1)*(x - 2)
+            return x * (x - 1) * (x - 2)
 
         # Test exceptions
         assert_raises(ValueError, lag.lagfit, [1], [1], -1)
@@ -460,7 +463,7 @@ class TestCompanion:
 
     def test_dimensions(self):
         for i in range(1, 5):
-            coef = [0]*i + [1]
+            coef = [0] * i + [1]
             assert_(lag.lagcompanion(coef).shape == (i, i))
 
     def test_linear_root(self):
@@ -477,7 +480,7 @@ class TestGauss:
         # functions like Laguerre can be very confusing.
         v = lag.lagvander(x, 99)
         vv = np.dot(v.T * w, v)
-        vd = 1/np.sqrt(vv.diagonal())
+        vd = 1 / np.sqrt(vv.diagonal())
         vv = vd[:, None] * vv * vd
         assert_almost_equal(vv, np.eye(100))
 
@@ -492,7 +495,7 @@ class TestMisc:
         res = lag.lagfromroots([])
         assert_almost_equal(trim(res), [1])
         for i in range(1, 5):
-            roots = np.cos(np.linspace(-np.pi, 0, 2*i + 1)[1::2])
+            roots = np.cos(np.linspace(-np.pi, 0, 2 * i + 1)[1::2])
             pol = lag.lagfromroots(roots)
             res = lag.lagval(roots, pol)
             tgt = 0
@@ -524,11 +527,11 @@ class TestMisc:
 
     def test_lag2poly(self):
         for i in range(7):
-            assert_almost_equal(lag.lag2poly([0]*i + [1]), Llist[i])
+            assert_almost_equal(lag.lag2poly([0] * i + [1]), Llist[i])
 
     def test_poly2lag(self):
         for i in range(7):
-            assert_almost_equal(lag.poly2lag(Llist[i]), [0]*i + [1])
+            assert_almost_equal(lag.poly2lag(Llist[i]), [0] * i + [1])
 
     def test_weight(self):
         x = np.linspace(0, 10, 11)
